/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004-2005 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Low-level functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//-------------------------------------------------------------------- CONSTANTS

static HMODULE hLastDLL = NULL;

//-------------------------------------------------------------------- VARIABLES

//----------------------------------------------------------- PRIVATE PROTOTYPES

//------------------------------------------------------------- PUBLIC FUNCTIONS

LPARAM wbSendMessage(PWBOBJ pwbo, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return 0;

	return SendMessage((HWND)pwbo->handle, uMsg, wParam, lParam);
}

BOOL wbSetWindowStyle(PWBOBJ pwbo, DWORD dwStyle, BOOL bExtended)
{
	SetLastError(0);	// See SetWindowLong() reference
	if(!SetWindowLong(pwbo->handle, GWL_STYLE, dwStyle) && GetLastError())
		return FALSE;
	if(!SetWindowLong(pwbo->handle, GWL_EXSTYLE, dwStyle) && GetLastError())
		return FALSE;
	else
		return TRUE;
}

/*
 Returns a handle to the library pszLibName. If pszLibName is NULL then uses last
 library called. Expands names until finds the library or returns NULL if no such
 library was found. The library pointer must be freed with wbReleaseLibrary.
*/

HMODULE wbLoadLibrary(LPCTSTR pszLibName)
{
	HMODULE hLib = NULL;
	char szName[256];

	if(!pszLibName || !*pszLibName)
		return NULL;

	// "DLL" is automatically appended by LoadLibrary()

	hLib = LoadLibrary(pszLibName);
	if(!hLib) {
		wsprintf(szName, "%s32", pszLibName);
		hLib = LoadLibrary(szName);
		if(!hLib) {
			wsprintf(szName, "%s.exe", pszLibName);
			hLib = LoadLibrary(szName);
			if(!hLib) {
				wsprintf(szName, "%s32.exe", pszLibName);
				hLib = LoadLibrary(szName);
			}
		}
	}

	hLastDLL = hLib;
	return hLib;
}

/*
 Returns a pointer to the library function pszFunction in the library pszLibName.
 Expands names until finds the function or returns NULL.
 Function names are limited to 255 characters.
*/

FARPROC wbGetLibraryFunction(HMODULE hLib, LPCTSTR pszFunction)
{
	FARPROC pFn;
	char szName[256];

	if(!hLib)
		if(hLastDLL)
			hLib = hLastDLL;

	if(!hLib)
		return NULL;

	// Try to obtain the function address "as is"

	pFn = GetProcAddress(hLib, pszFunction);
	if(pFn)
		return pFn;

	// Try appending an 'A' to the function name

	wsprintf(szName, "%sA", pszFunction);
	pFn = GetProcAddress(hLib, szName);
	if(pFn)
		return pFn;

	// Try appending an 'W' to the function name

	wsprintf(szName, "%sW", pszFunction);
	pFn = GetProcAddress(hLib, szName);
	if(pFn)
		return pFn;

	// Nothing else to try

	return NULL;
}

/* Releases the library handle hLib. hLib must have been obtained with wbLoadLibrary. */

BOOL wbReleaseLibrary(HMODULE hLib)
{
	if(hLib) {
		FreeLibrary(hLib);
		hLastDLL = NULL;
		return TRUE;
	} else {
		return FALSE;
	}
}

//------------------------------------------------------------------ END OF FILE
